import json
import os
from typing import Any, Callable, Optional, Union

from consul import Consul


def load_config_from_consul(  # pylint: disable=too-many-arguments
        host: str,
        port: int,
        token: str,
        root_key: str,
        app_key: str,
        env_key: str,
        common_key: Optional[str] = None,
        value_deserializer=json.loads,
        deserializer: Callable = lambda x: x,
        **kwargs,
) -> Union[object, dict]:
    """
    Load config from ``consul`` server.

    It expects the following key structure on ``consul`` side

    <root-key>
        |____<common-config-key>
        |          |
        |          |___<some-env-key>
        |          |           |_____<key-value>
        |          |           |_____<key-value>
        |          |
        |          |___<another-env-key>
        |                      |_____<key-value>
        |                      |_____<key-value>
        |____<app-config-key>
                   |
                   |___<some-env-key>
                   |           |_____<key-value>
                   |           |_____<key-value>
                   |
                   |___<another-env-key>
                               |_____<key-value>
                               |_____<key-value>


    :param str host: host of consule server
    :param int port:
    :param str root_key:
    :param str app_key:
    :param str env_key:
    :param str common_key:
    """
    server = Consul(host, port, token, **kwargs)
    common_config = dict()
    if common_key:
        common_config = _get_config_for_keys_from_consul(
            server,
            f'{root_key}/{common_key}/{env_key}/',
            deserializer=value_deserializer,
        )

    app_config = _get_config_for_keys_from_consul(
        server,
        f'{root_key}/{app_key}/{env_key}/',
        deserializer=value_deserializer,
    )
    configs = deserializer({**common_config, **app_config})
    return configs


def _get_config_for_keys_from_consul(
        server: Consul,
        key: str,
        deserializer: Callable,
) -> dict:
    _, raw_configs = server.kv.get(key, recurse=True)
    raw_configs = raw_configs[1:] if raw_configs else list()
    return {
        config['Key'].lstrip(key): deserializer(config['Value'])
        for config in raw_configs
    }


def _set_environment_variables(config: dict, prefix: str = '') -> None:
    for key, value in config.items():
        os.environ[f'{prefix}{key}'] = _serialize_value(value)


def _serialize_value(
        value: Any,
        serializer: Callable = lambda x: json.dumps(x)
        if isinstance(x, (dict, list)) else str(x)
) -> str:
    return serializer(value)



import sys
from pprint import pformat

from rconfig import (
    _serialize_value, _set_environment_variables, load_config_from_consul,
)


try:
    import click
except ImportError:
    sys.stderr.write('Missing cli packages.\nRun pip install "rconfig[cli]"')


@click.group()
@click.option('-h', '--host', required=True, help='Host of consul server')
@click.option(
    '-p', '--port', default=8500, type=click.INT, help='Port of consul server',
)
@click.option(
    '-a', '--access', required=True, help='Access key to consul server',
)
@click.option('--root', required=True, help='Root key')
@click.option('--app', required=True, help='Application key')
@click.option('--env', required=True, help='Environment key')
@click.option('--common', default='common', help='Common key')
@click.pass_context
def cli(ctx, host, port, access, root, app, env, common):
    ctx.obj = dict()
    ctx.obj['HOST'] = host
    ctx.obj['PORT'] = port
    ctx.obj['ACCESS'] = access
    ctx.obj['ROOT'] = root
    ctx.obj['APP'] = app
    ctx.obj['ENV'] = env
    ctx.obj['COMMON'] = common


@cli.command()
@click.pass_context
def list(ctx):  # pylint: disable=redefined-builtin
    "Show all config for given keys"
    host = ctx.obj['HOST']
    port = ctx.obj['PORT']
    access = ctx.obj['ACCESS']
    root = ctx.obj['ROOT']
    app = ctx.obj['APP']
    env = ctx.obj['ENV']
    common = ctx.obj['COMMON']
    config = load_config_from_consul(
        host, port, access, root, app, env, common,
    )
    if config:
        click.echo(pformat(config))
    else:
        click.echo('No config for given keys')
        sys.exit(1)


@cli.command()
@click.pass_context
@click.option('--prefix', default='', help='Prefix for environment keys')
def set_envs(ctx, prefix):  # pylint: disable=R0913
    "Set envs in current subprocess"
    host = ctx.obj['HOST']
    port = ctx.obj['PORT']
    access = ctx.obj['ACCESS']
    root = ctx.obj['ROOT']
    app = ctx.obj['APP']
    env = ctx.obj['ENV']
    common = ctx.obj['COMMON']
    config = \
        load_config_from_consul(host, port, access, root, app, env, common)
    if config:
        _set_environment_variables(config, prefix)
    else:
        click.echo('No config for given keys')
        sys.exit(1)


@cli.command()
@click.pass_context
@click.option('--prefix', default='', help='Prefix for environment keys')
def export(ctx, prefix):  # pylint: disable=R0913
    "Print out bash command export for all found config"
    host = ctx.obj['HOST']
    port = ctx.obj['PORT']
    access = ctx.obj['ACCESS']
    root = ctx.obj['ROOT']
    app = ctx.obj['APP']
    env = ctx.obj['ENV']
    common = ctx.obj['COMMON']
    config = load_config_from_consul(
        host, port, access, root, app, env, common,
    )
    envs = ' '.join(
        f'{prefix}{key}="{_serialize_value(value)}"'
        for key, value in config.items()
    )
    if not envs:
        sys.exit(1)
    click.echo(f'export {envs}')


if __name__ == '__main__':
    cli()  # pylint: disable=no-value-for-parameter



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['rconfig']

package_data = \
{'': ['*']}

install_requires = \
['python-consul>=1.1.0,<2.0.0']

extras_require = \
{'cli': ['click>=7.0,<8.0']}

entry_points = \
{'console_scripts': ['rconfig = rconfig.cli:cli']}

setup_kwargs = {
    'name': 'python-rconfig',
    'version': '20.0.1',
    'description': 'Helps get configuration from consul server',
    'long_description': 'rconfig\n=======\n\n  .. image:: https://travis-ci.org/ArtemAngelchev/python-rconfig.svg?branch=master\n      :target: https://travis-ci.org/ArtemAngelchev/python-rconfig\n\n  .. image:: http://img.shields.io/badge/license-MIT-brightgreen.svg?style=flat\n\n\n  ``rconfig`` helps you to bring configs from a consul server to your\n  application.\n\n\nInstallation\n------------\n\n  Install the latest version with:\n\n  ::\n\n    pip install -U python-rconfig\n\n\n  For command-line support, use the CLI option during installation:\n\n  ::\n\n    pip install -U "python-rconfig[cli]"\n\n\nUsage\n-----\n\n  First off all ``rconfig`` expects that you have the following key structure\n  on the consul server:\n\n  ::\n\n    <root-key>\n        |____<common-config-key>\n        |          |\n        |          |___<some-env-key>\n        |          |           |_____<key-value>\n        |          |           |_____<key-value>\n        |          |\n        |          |___<another-env-key>\n        |                      |_____<key-value>\n        |                      |_____<key-value>\n        |____<app-config-key>\n                   |\n                   |___<some-env-key>\n                   |           |_____<key-value>\n                   |           |_____<key-value>\n                   |\n                   |___<another-env-key>\n                               |_____<key-value>\n                               |_____<key-value>\n\n\n  Here root key stands for the name of the project when some have multiple\n  applications that grouped under some kind of common purpose (often when talk\n  about microservices).\n  Under common configuration key, you should store configurations that common\n  to all your applications in the project, in this case, it\'s much easier to\n  change the config in one place than go to multiple.\n',
    'author': 'Artem Angelchev',
    'author_email': 'artangelchev@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/ArtemAngelchev/python-rconfig',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'entry_points': entry_points,
    'python_requires': '>=3.6,<4',
}


setup(**setup_kwargs)
